var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var _map, _default;
class ObjectMap {
    constructor(defaultValue) {
        _map.set(this, new WeakMap());
        _default.set(this, void 0);
        __classPrivateFieldSet(this, _default, defaultValue);
    }
    // eslint-disable-next-line @typescript-eslint/ban-types
    get(object, property) {
        if (!__classPrivateFieldGet(this, _map).has(object)) {
            __classPrivateFieldGet(this, _map).set(object, new Map());
        }
        const map = __classPrivateFieldGet(this, _map).get(object);
        if (!map.has(property)) {
            map.set(property, __classPrivateFieldGet(this, _default).call(this));
        }
        return map.get(property);
    }
    // eslint-disable-next-line @typescript-eslint/ban-types
    set(object, property, value) {
        if (!__classPrivateFieldGet(this, _map).has(object)) {
            __classPrivateFieldGet(this, _map).set(object, new Map());
        }
        __classPrivateFieldGet(this, _map).get(object).set(property, value);
    }
}
_map = new WeakMap(), _default = new WeakMap();
const observed = new ObjectMap(() => undefined);
const observers = new ObjectMap(() => new Set());
export function observeProperty(object, property, updateCallback) {
    if (!observers.get(object, property).size) {
        const descriptor = Object.getOwnPropertyDescriptor(object, property);
        if (descriptor && !descriptor.configurable) {
            throw new Error(`Cannot observe non-configurable property ${property}`);
        }
        const proto = Object.getPrototypeOf(object);
        const protoDescriptor = proto && Object.getOwnPropertyDescriptor(proto, property);
        if (descriptor && 'value' in descriptor)
            observed.set(object, property, descriptor.value);
        const get = (descriptor && descriptor.get) || (protoDescriptor && protoDescriptor.get) || null;
        const set = (descriptor && descriptor.set) || (protoDescriptor && protoDescriptor.set) || null;
        Object.defineProperty(object, property, {
            configurable: true,
            get() {
                if (get) {
                    const newValue = get.call(this);
                    const oldValue = observed.get(this, property);
                    if (!Object.is(oldValue, newValue)) {
                        observed.set(this, property, newValue);
                        for (const callback of observers.get(this, property))
                            callback(newValue, oldValue);
                    }
                }
                return observed.get(this, property);
            },
            set(newValue) {
                const oldValue = this[property];
                if (set) {
                    set.call(this, newValue);
                    if (get)
                        newValue = get.call(this);
                }
                if (!Object.is(oldValue, newValue)) {
                    observed.set(this, property, newValue);
                    for (const callback of observers.get(this, property))
                        callback(newValue, oldValue);
                }
            }
        });
    }
    observers.get(object, property).add(updateCallback);
}
//# sourceMappingURL=observe-property.js.map